searchData={"items":[{"type":"module","title":"SweetXml","doc":"`SweetXml` is a thin wrapper around `:xmerl`. It allows you to convert a\nstring or xmlElement record as defined in `:xmerl` to an elixir value such\nas `map`, `list`, `char_list`, or any combination of these.\n\nFor normal sized documents, `SweetXml` primarily exposes 3 functions\n\n  * `SweetXml.xpath/2` - return a value based on the xpath expression\n  * `SweetXml.xpath/3` - similar to above but allowing nesting of mapping\n  * `SweetXml.xmap/2` - return a map with keywords mapped to values returned\n    from xpath\n\nFor something larger, `SweetXml` mainly exposes 1 function\n\n  * `SweetXml.stream_tags/3` - stream a given tag or a list of tags, and\n    optionally \"discard\" some dom elements in order to free memory during\n    streaming for big files which cannot fit entirely in memory","ref":"SweetXml.html"},{"type":"module","title":"Examples - SweetXml","doc":"Simple Xpath:\n\n    iex> import SweetXml\n    iex> doc = \"  Some linked title  \"\n    iex> doc |> xpath(~x\"//a/text()\")\n    'Some linked title'\n\nNested Mapping:\n\n    iex> import SweetXml\n    iex> doc = \"   Message   One   Two     \"\n    iex> doc |> xpath(~x\"//header\", message: ~x\"./p/text()\", a_in_li: ~x\".//li/a/text()\"l)\n    %{a_in_li: ['Two'], message: 'Message'}\n\nStreaming:\n\n    iex> import SweetXml\n    iex> doc = [\"  l1  l2\", \"  l3  \"]\n    iex> SweetXml.stream_tags(doc, :li)\n    ...> |> Stream.map(fn {:li, doc} ->\n    ...>      doc |> SweetXml.xpath(~x\"./text()\")\n    ...>    end)\n    ...> |> Enum.to_list\n    ['l1', 'l2', 'l3']\n\nFor more examples please see help for each individual functions","ref":"SweetXml.html#module-examples"},{"type":"module","title":"The ~x Sigil - SweetXml","doc":"Warning ! Because we use `xmerl` internally, only XPath 1.0 paths are handled.\n\nNotice in the above examples, we used the expression `~x\"//a/text()\"` to\ndefine the path. The reason is it allows us to more precisely specify what\nis being returned.\n\n  * `~x\"//some/path\"`\n\n    without any modifiers, `xpath/2` will return the value of the entity if\n    the entity is of type `xmlText`, `xmlAttribute`, `xmlPI`, `xmlComment`\n    as defined in `:xmerl`\n\n  * `~x\"//some/path\"e`\n\n    `e` stands for (e)ntity. This forces `xpath/2` to return the entity with\n    which you can further chain your `xpath/2` call\n\n  * `~x\"//some/path\"l`\n\n    'l' stands for (l)ist. This forces `xpath/2` to return a list. Without\n    `l`, `xpath/2` will only return the first element of the match\n\n  * `~x\"//some/path\"el` - mix of the above\n\n  * `~x\"//some/path\"k`\n\n    'k' stands for (K)eyword. This forces `xpath/2` to return a Keyword instead of a Map.\n\n  * `~x\"//some/path\"s`\n\n    's' stands for (s)tring. This forces `xpath/2` to return the value as\n    string instead of a char list.\n\n  * `x\"//some/path\"o`\n\n    'o' stands for (O)ptional. This allows the path to not exist, and will return nil.\n\n  * `~x\"//some/path\"sl` - string list.\n\nNotice also in the examples section, we always import SweetXml first. This\nmakes `x_sigil` available in the current scope. Without it, instead of using\n`~x`, you can do the following\n\n    iex> doc = \"  Some linked title  \"\n    iex> doc |> SweetXml.xpath(%SweetXpath{path: '//a/text()', is_value: true, cast_to: false, is_list: false, is_keyword: false})\n    'Some linked title'\n\nNote the use of char_list in the path definition.","ref":"SweetXml.html#module-the-x-sigil"},{"type":"function","title":"SweetXml.add_namespace/3","doc":"","ref":"SweetXml.html#add_namespace/3"},{"type":"type","title":"SweetXml.doc/0","doc":"","ref":"SweetXml.html#t:doc/0"},{"type":"function","title":"SweetXml.parse/2","doc":"Parse a document into a form ready to be used by `xpath/3` and `xmap/2`.\n\n`doc` can be\n\n- a byte list (iodata)\n- a binary\n- any enumerable of binaries (for instance `File.stream!/3` result)\n\n`options` can be both:\n* `xmerl`'s options as described on the [xmerl_scan](http://www.erlang.org/doc/man/xmerl_scan.html) documentation page,\n  see [the erlang tutorial](http://www.erlang.org/doc/apps/xmerl/xmerl_examples.html) for some advanced usage.\n    For example: `parse(doc, quiet: true)`\n* `:dtd` to prevent DTD parsing or fetching, with the following possibilities:\n  * `:none`, will prevent both internal and external entities, it is the recommended options on untrusted XML.\n    This will override the option `{:rules, read_fun, write_fun, state}` if present;\n  * `:all`, the default, for backward compatibility, allows all DTDs;\n  * `:internal_only`, will block all attempt at external fetching;\n  * `[only: entities]` where `entities` is either an atom for a single entity, or a list of atoms.\n    If any other entity is defined in the XML, `parse` will raise on them.\n    This will override the option `{:rules, read_fun, write_fun, state}` if present.\n\nWhen `doc` is an enumerable, the `:cont_fun` option cannot be given.\n\nReturns an `xmlElement` record.","ref":"SweetXml.html#parse/2"},{"type":"function","title":"SweetXml.sigil_x/2","doc":"`sigil_x/2` simply returns a `%SweetXpath{}` struct, with modifiers converted to\nboolean fields:\n\n    iex> SweetXml.sigil_x(\"//some/path\", 'e')\n    %SweetXpath{path: '//some/path', is_value: false, cast_to: false, is_list: false, is_keyword: false}\n\nOr you can simply import and use the `~x` expression:\n\n    iex> import SweetXml\n    iex> ~x\"//some/path\"e\n    %SweetXpath{path: '//some/path', is_value: false, cast_to: false, is_list: false, is_keyword: false}\n\nValid modifiers are `e`, `s`, `l` and `k`. Below is the full explanation\n\n  * `~x\"//some/path\"`\n\n    without any modifiers, `xpath/2` will return the value of the entity if\n    the entity is of type `xmlText`, `xmlAttribute`, `xmlPI`, `xmlComment`\n    as defined in `:xmerl`\n\n  * `~x\"//some/path\"e`\n\n    `e` stands for (e)ntity. This forces `xpath/2` to return the entity with\n    which you can further chain your `xpath/2` call\n\n  * `~x\"//some/path\"l`\n\n    'l' stands for (l)ist. This forces `xpath/2` to return a list. Without\n    `l`, `xpath/2` will only return the first element of the match\n\n  * `~x\"//some/path\"el` - mix of the above\n\n  * `~x\"//some/path\"k`\n\n    'k' stands for (K)eyword. This forces `xpath/2` to return a Keyword instead of a Map.\n\n  * `~x\"//some/path\"s`\n\n    's' stands for (s)tring. This forces `xpath/2` to return the value as\n    string instead of a char list.\n\n  * `x\"//some/path\"o`\n\n    'o' stands for (O)ptional. This allows the path to not exist, and will return nil.\n\n  * `~x\"//some/path\"sl` - string list.\n\n  * `~x\"//some/path\"i`\n\n    'i' stands for (i)nteger. This forces `xpath/2` to return the value as\n    integer instead of a char list.\n\n  * `~x\"//some/path\"f`\n\n    'f' stands for (f)loat. This forces `xpath/2` to return the value as\n    float instead of a char list.\n\n  * `~x\"//some/path\"il` - integer list","ref":"SweetXml.html#sigil_x/2"},{"type":"type","title":"SweetXml.spec/0","doc":"","ref":"SweetXml.html#t:spec/0"},{"type":"function","title":"SweetXml.stream/2","doc":"> #### Soft Deprecation {: .warning}\n>\n> Will be later deprecated in favor of `stream!/2`.\n\nCreate an element stream from a XML `doc`.\n\nThis is a lower level API compared to `SweetXml.stream_tags`. You can use\nthe `options_callback` argument to get fine control of what data to be streamed.\n\n- `doc` is an enumerable, data will be pulled during the result stream\n  enumeration. e.g. `File.stream!(\"some_file.xml\")`\n- `options_callback` is an anonymous function `fn emit -> (xmerl_opts | opts)` use it to\n  define your :xmerl callbacks and put data into the stream using\n  `emit.(elem)` in the callbacks. More details are available with `parse/2`.\n\nFor example, here you define a stream of all `xmlElement` :\n\n    iex> import Record\n    iex> doc = [\"  Som\", \"e linked title  other  \"]\n    iex> SweetXml.stream(doc, fn emit ->\n    ...>   [\n    ...>     hook_fun: fn\n    ...>       entity, xstate when is_record(entity, :xmlElement)->\n    ...>         emit.(entity)\n    ...>         {entity, xstate}\n    ...>       entity, xstate ->\n    ...>         {entity,xstate}\n    ...>     end\n    ...>   ]\n    ...> end) |> Enum.count\n    3","ref":"SweetXml.html#stream/2"},{"type":"function","title":"SweetXml.stream!/2","doc":"Equivalent to `stream/2`, see `stream/2` for more details.\nThe difference is in the handling of the errors. The caller can use `try/1`,\nwhereas with `stream/3` trapping exits and handling messages was necessary.\nMay raise `SweetXml.XmerlFatal` or `SweetXml.DTDError`.","ref":"SweetXml.html#stream!/2"},{"type":"function","title":"SweetXml.stream_tags/3","doc":"> #### Soft Deprecation {: .warning}\n>\n> Will be later deprecated in favor of `stream_tags!/3`.\n\nMost common usage of streaming: stream a given tag or a list of tags, and\noptionally \"discard\" some DOM elements in order to free memory during streaming\nfor big files which cannot fit entirely in memory.\n\nNote that each matched tag produces it's own tree. If a given tag appears in\nthe discarded options, it is ignored.\n\n- `doc` is an enumerable, data will be pulled during the result stream\n  enumeration. e.g. `File.stream!(\"some_file.xml\")`\n- `tags` is an atom or a list of atoms you want to extract. Each stream element\n  will be `{:tagname, xmlelem}`. e.g. :li, :header\n- `options[:discard]` is the list of tag which will be discarded:\n   not added to its parent DOM.\n- More options details are available with `parse/2`.","ref":"SweetXml.html#stream_tags/3"},{"type":"function","title":"Examples - SweetXml.stream_tags/3","doc":"iex> import SweetXml\n    iex> doc = [\"  l1  l2\", \"  l3  \"]\n    iex> SweetXml.stream_tags(doc, :li, discard: [:li])\n    ...> |> Stream.map(fn {:li, doc} -> doc |> SweetXml.xpath(~x\"./text()\") end)\n    ...> |> Enum.to_list\n    ['l1', 'l2', 'l3']\n    iex> SweetXml.stream_tags(doc, [:ul, :li])\n    ...> |> Stream.map(fn {_, doc} -> doc |> SweetXml.xpath(~x\"./text()\") end)\n    ...> |> Enum.to_list\n    ['l1', 'l2', 'l3', nil]\n\n\nBe careful if you set `options[:discard]`. If any of the discarded tags is nested\ninside a kept tag, you will not be able to access them.","ref":"SweetXml.html#stream_tags/3-examples"},{"type":"function","title":"Examples - SweetXml.stream_tags/3","doc":"iex> import SweetXml\n    iex> doc = [\" \", \" XML   Nested   \"]\n    iex> SweetXml.stream_tags(doc, :header)\n    ...> |> Stream.map(fn {_, doc} -> SweetXml.xpath(doc, ~x\".//title/text()\") end)\n    ...> |> Enum.to_list\n    ['Nested', 'XML']\n    iex> SweetXml.stream_tags(doc, :header, discard: [:title])\n    ...> |> Stream.map(fn {_, doc} -> SweetXml.xpath(doc, ~x\"./title/text()\") end)\n    ...> |> Enum.to_list\n    [nil, nil]","ref":"SweetXml.html#stream_tags/3-examples"},{"type":"function","title":"SweetXml.stream_tags!/3","doc":"Equivalent to `stream_tags/3`, see `stream_tags/3` for more details.\nThe difference is in the handling of the errors. The caller can use `try/1`,\nwhereas with `stream_tags/3` trapping exits and handling messages was necessary.\nMay raise `SweetXml.XmerlFatal` or `SweetXml.DTDError`.","ref":"SweetXml.html#stream_tags!/3"},{"type":"function","title":"SweetXml.transform_by/2","doc":"Tags `%SweetXpath{}` with `fun` to be applied at the end of `xpath` query.","ref":"SweetXml.html#transform_by/2"},{"type":"function","title":"Examples - SweetXml.transform_by/2","doc":"iex> import SweetXml\n    iex> string_to_range = fn str ->\n    ...>     [first, last] = str |> String.split(\"-\", trim: true) |> Enum.map(&String.to_integer/1)\n    ...>     first..last\n    ...>   end\n    iex> doc = \"   north  5-15   \"\n    iex> doc\n    ...> |> xpath(\n    ...>      ~x\"//weather/zone\"l,\n    ...>      name: ~x\"//name/text()\"s |> transform_by(&String.capitalize/1),\n    ...>      wind_speed: ~x\"./wind-speed/text()\"s |> transform_by(string_to_range)\n    ...>    )\n    [%{name: \"North\", wind_speed: 5..15}]","ref":"SweetXml.html#transform_by/2-examples"},{"type":"function","title":"SweetXml.xmap/3","doc":"`xmap` returns a mapping with each value being the result of `xpath`.\n\nJust as `xpath`, you can nest the mapping structure. Please see `xpath/3` for\nmore detail.\n\nYou can give the option `true` to get the result as a keyword list instead of a map.","ref":"SweetXml.html#xmap/3"},{"type":"function","title":"Examples - SweetXml.xmap/3","doc":"Simple:\n\n    iex> import SweetXml\n    iex> doc = \"  Some linked title  \"\n    iex> doc |> xmap(a: ~x\"//a/text()\")\n    %{a: 'Some linked title'}\n\nWith optional mapping:\n\n    iex> import SweetXml\n    iex> doc = \"   Message   One   Two     \"\n    iex> doc |> xmap(message: ~x\"//p/text()\", a_in_li: ~x\".//li/a/text()\"l)\n    %{a_in_li: ['Two'], message: 'Message'}\n\nWith optional mapping and nesting:\n\n    iex> import SweetXml\n    iex> doc = \"   Message   One   Two     \"\n    iex> doc\n    ...> |> xmap(\n    ...>      message: ~x\"//p/text()\",\n    ...>      ul: [\n    ...>        ~x\"//ul\",\n    ...>        a: ~x\"./li/a/text()\"\n    ...>      ]\n    ...>    )\n    %{message: 'Message', ul: %{a: 'Two'}}\n    iex> doc\n    ...> |> xmap(\n    ...>      message: ~x\"//p/text()\",\n    ...>      ul: [\n    ...>        ~x\"//ul\"k,\n    ...>        a: ~x\"./li/a/text()\"\n    ...>      ]\n    ...>    )\n    %{message: 'Message', ul: [a: 'Two']}\n    iex> doc\n    ...> |> xmap([\n    ...>      message: ~x\"//p/text()\",\n    ...>      ul: [\n    ...>        ~x\"//ul\",\n    ...>        a: ~x\"./li/a/text()\"\n    ...>      ]\n    ...>    ], true)\n    [message: 'Message', ul: %{a: 'Two'}]","ref":"SweetXml.html#xmap/3-examples"},{"type":"function","title":"Security - SweetXml.xmap/3","doc":"Whenever you are working with some xml that was not generated by your system,\nit is highly recommended that you restrain some functionalities of XML\nduring the parsing. SweetXml allows in particular to prevent DTD parsing and fetching.\nUnless you know exactly what kind of DTD you want to permit in your xml,\nit is recommended that you use the following code example to prevent possible attacks:\n```\ndoc\n|> parse(dtd: :none)\n|> xmap(specs, options)\n```\nFor more details, see `parse/2`.","ref":"SweetXml.html#xmap/3-security"},{"type":"macro","title":"SweetXml.xmlAttribute/2","doc":"","ref":"SweetXml.html#xmlAttribute/2"},{"type":"macro","title":"SweetXml.xmlComment/2","doc":"","ref":"SweetXml.html#xmlComment/2"},{"type":"macro","title":"SweetXml.xmlDecl/2","doc":"","ref":"SweetXml.html#xmlDecl/2"},{"type":"macro","title":"SweetXml.xmlDocument/2","doc":"","ref":"SweetXml.html#xmlDocument/2"},{"type":"type","title":"SweetXml.xmlElement/0","doc":"","ref":"SweetXml.html#t:xmlElement/0"},{"type":"macro","title":"SweetXml.xmlElement/2","doc":"","ref":"SweetXml.html#xmlElement/2"},{"type":"macro","title":"SweetXml.xmlNamespace/2","doc":"","ref":"SweetXml.html#xmlNamespace/2"},{"type":"macro","title":"SweetXml.xmlNsNode/2","doc":"","ref":"SweetXml.html#xmlNsNode/2"},{"type":"macro","title":"SweetXml.xmlObj/2","doc":"","ref":"SweetXml.html#xmlObj/2"},{"type":"macro","title":"SweetXml.xmlPI/2","doc":"","ref":"SweetXml.html#xmlPI/2"},{"type":"macro","title":"SweetXml.xmlText/2","doc":"","ref":"SweetXml.html#xmlText/2"},{"type":"function","title":"SweetXml.xpath/3","doc":"`xpath` allows you to query an XML document with XPath.\n\nThe second argument to xpath is a `%SweetXpath{}` struct. The optional third\nargument is a keyword list, such that the value of each keyword is also\neither a `%SweetXpath{}` or a list with head being a `%SweetXpath{}` and tail being\nanother keyword list exactly like before. Please see the examples below for better\nunderstanding.","ref":"SweetXml.html#xpath/3"},{"type":"function","title":"Examples - SweetXml.xpath/3","doc":"Simple:\n\n    iex> import SweetXml\n    iex> doc = \"  Some linked title  \"\n    iex> doc |> xpath(~x\"//a/text()\")\n    'Some linked title'\n\nWith optional mapping:\n\n    iex> import SweetXml\n    iex> doc = \"   Message   One   Two     \"\n    iex> doc |> xpath(~x\"//header\", message: ~x\"./p/text()\", a_in_li: ~x\".//li/a/text()\"l)\n    %{a_in_li: ['Two'], message: 'Message'}\n\nWith optional mapping and nesting:\n\n    iex> import SweetXml\n    iex> doc = \"   Message   One   Two     \"\n    iex> doc\n    ...> |> xpath(\n    ...>      ~x\"//header\",\n    ...>      ul: [\n    ...>        ~x\"./ul\",\n    ...>        a: ~x\"./li/a/text()\"\n    ...>      ]\n    ...>    )\n    %{ul: %{a: 'Two'}}","ref":"SweetXml.html#xpath/3-examples"},{"type":"function","title":"Security - SweetXml.xpath/3","doc":"Whenever you are working with some xml that was not generated by your system,\nit is highly recommended that you restrain some functionalities of XML\nduring the parsing. SweetXml allows in particular to prevent DTD parsing and fetching.\nUnless you know exactly what kind of DTD you want to permit in your xml,\nit is recommended that you use the following code example to prevent possible attacks:\n```\ndoc\n|> parse(dtd: :none)\n|> xpath(spec, subspec)\n```\nFor more details, see `parse/2`.","ref":"SweetXml.html#xpath/3-security"},{"type":"exception","title":"SweetXml.DTDError","doc":"An error raised when a non allowed DTD is encountered.","ref":"SweetXml.DTDError.html"},{"type":"exception","title":"SweetXml.XmerlFatal","doc":"An error raised when xmerl exits with a fatal error.","ref":"SweetXml.XmerlFatal.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [0.7.5] (2025-01-07)\nA maintenance patch, with improvements made the the README, with up to date installation\ninstructions from [@arcanemachine](https://github.com/arcanemachine) ([#95](https://github.com/kbrw/sweet_xml/pull/95))\nand typos fixed by [@kianmeng](https://github.com/kianmeng) ([#89](https://github.com/kbrw/sweet_xml/pull/89)).","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"* Warning cleanups removal made by [@VictorTrin](https://github.com/VitorTrin) and [@elfenlaid](https://github.com/elfenlaid)\n([#96](https://github.com/kbrw/sweet_xml/pull/96) & [#102](https://github.com/kbrw/sweet_xml/pull/102))\n\n## [0.7.4] (2023-08-31)","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* Contribution from [@thbar](https://github.com/thbar):\n  * Logger deprecation warning fixed for Elixir 1.15 (`#94`)\n  * Now requires Elixir 1.12+ (`#94`)\n\n## [0.7.3] (2022-04-11)","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* Handling the option `{:rules, _}` ourselves, as well as partially the option `{:rules, _, _, _}`.\n  When nothing is given, a new table is created, and destroyed after.\n  If a table is given, and a DTD option is chosen, the table is reused for the DTD handling.\n  If a custom `{:rules, _, _, _}` is given, and a restrictive DTD option is chosen, the custome rule will be overridden.\n  Because of this mutual exclusivity (reconciling the behaviors is not possible via composition), it is recommended that you handle the DTD\n  problem by yourself. You can see the issue `#71` for ideas.\n  See the issue `#41` for more details on why this change happened. (Ets leaks.)\n\n## [0.7.2] (2021-11-25)","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* Improvement on the doc, notably warning that only XPath 1.0 queries are handled.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"* Contribution from [@tank-bohr](https://github.com/github/tank-bohr): Added strict stream API with a more coherent overall behavior, and easier to handle failure behavior:\n  `stream!/2` and `stream_tags!/2,3` are now available.\n\n## [0.7.1] (2021-08-25)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* [@J3RN](https://github.com/J3RN) noticed a breaking change due to the introduction of an opaque type  \n  Changed `xmlElement` from `@opaque` to `@type`\n\n## [0.7.0] (2021-07-02)","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* Improvement on the doc","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"* Added option to raise on DTD definitions\n\n## [0.6.6] (2019-02-24)\n\n* small bugfix: Fix compilation warnings on newer versions of Elixir\n* doc updates\n\n\n## [0.6.5] (2017-02-08)\n\n* small bugfix : unexpected `:halted` of enumerable\n\n## [0.6.4] (2017-01-17)\n\n* make it compatible with Elixir 1.4\n\n## [0.6.3] (2016-12-14)\n\n* bugfix: xpath `optional` with cast and void should return nil\n* add \"soft\" uppercase cast modifier : put default value if type\n  modifier is uppercase \"\",0,0.0\n* if \"soft\" and \"optional\", then return nil with value not compatible\n\n## [0.6.1] (2016-02-10)\n\n* bugfix: xpath `list` modifier should always return a list\n\n## [0.6.0] (2016-02-09)\n\n* text() XML nodes are now joined with the `s` modifier\n* `transform_by` function allows you to customize each mapped field\n  with any function\n\n## [0.5.1] (2016-02-08)\n\n## [0.5.0] (2015-10-28)\n\n* Added support to optional modifier\n* Update Elixir requirement\n* Map refactoring, from if/else tree to cleaner pattern matching\n* Add casting capabilities\n* Fix dependencies : doc deps only for `:dev` and OTP `:xmerl` dependency\n\n## [0.4.0] (2015-09-11)\n\n* Added support to map into keyword list instead of maps\n\n## [0.3.0] (2015-08-10)\n\n* Added support to return values as strings (viniciussbs)\n\n* Fix stream `:halt` handling (awetzel)\n\n## [0.2.1] (2016-04-12)\n\n## [0.2.0] (2016-04-11)\n\n* Fixed encoding issue and improved speed (awetzel)\n\n* Added file streaming support (awetzel)\n\n* Added element streaming (awetzel)\n\n* Added support for scalar values (xbrukner)\n\n## [0.1.1]\n\n---\n\nChangelog format inspired by [keep-a-changelog](https://github.com/olivierlacan/keep-a-changelog)\n\n[unreleased]: https://github.com/kbrw/sweet_xml/compare/v0.7.5...HEAD\n\n[0.7.5]: https://github.com/kbrw/sweet_xml/compare/v0.7.4...v0.7.5\n[0.7.4]: https://github.com/kbrw/sweet_xml/compare/v0.7.3...v0.7.4\n[0.7.3]: https://github.com/kbrw/sweet_xml/compare/v0.7.2...v0.7.3\n[0.7.2]: https://github.com/kbrw/sweet_xml/compare/v0.7.1...v0.7.2\n[0.7.1]: https://github.com/kbrw/sweet_xml/compare/v0.7.0...v0.7.1\n[0.7.0]: https://github.com/kbrw/sweet_xml/compare/0.6.6...v0.7.0\n[0.6.6]: https://github.com/kbrw/sweet_xml/compare/0.6.5...0.6.6\n[0.6.5]: https://github.com/kbrw/sweet_xml/compare/0.6.4...0.6.5\n[0.6.4]: https://github.com/kbrw/sweet_xml/compare/0.6.3...0.6.4\n[0.6.3]: https://github.com/kbrw/sweet_xml/compare/0.6.2...0.6.3\n[0.6.2]: https://github.com/kbrw/sweet_xml/compare/0.6.1...0.6.2\n[0.6.1]: https://github.com/kbrw/sweet_xml/compare/0.6.0...0.6.1\n[0.6.0]: https://github.com/kbrw/sweet_xml/compare/0.5.1...0.6.0\n[0.5.1]: https://github.com/kbrw/sweet_xml/compare/0.5.0...0.5.1\n[0.5.0]: https://github.com/kbrw/sweet_xml/compare/0.4.0...0.5.0\n[0.4.0]: https://github.com/kbrw/sweet_xml/compare/0.3.0...0.4.0\n[0.3.0]: https://github.com/kbrw/sweet_xml/compare/0.2.1...0.3.0\n[0.2.1]: https://github.com/kbrw/sweet_xml/compare/0.2.0...0.2.1\n[0.2.0]: https://github.com/kbrw/sweet_xml/compare/0.1.1...0.2.0\n[0.1.1]: https://github.com/kbrw/sweet_xml/compare/f203bdf...0.1.1","ref":"changelog.html#added"},{"type":"extras","title":"Overview","doc":"# SweetXml\n\n[![Build Status]](https://github.com/kbrw/sweet_xml/actions/workflows/.github/workflows/build-and-test.yml/badge.svg)\n[![Module Version](https://img.shields.io/hexpm/v/sweet_xml.svg)](https://hex.pm/packages/sweet_xml)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/sweet_xml/)\n[![Total Download](https://img.shields.io/hexpm/dt/sweet_xml.svg)](https://hex.pm/packages/sweet_xml)\n[![License](https://img.shields.io/hexpm/l/sweet_xml.svg)](https://github.com/kbrw/sweet_xml/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/kbrw/sweet_xml.svg)](https://github.com/kbrw/sweet_xml/commits/master)\n\n`SweetXml` is a thin wrapper around `:xmerl`. It allows you to convert a\n`char_list` or `xmlElement` record as defined in `:xmerl` to an elixir value such\nas `map`, `list`, `string`, `integer`, `float` or any combination of these.","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Add dependency to your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [{:sweet_xml, \"~> 0.7.5\"}]\nend\n```\n\n`SweetXml` depends on `:xmerl`. On some Linux systems, you might need\nto install the package `erlang-xmerl`.","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Overview","doc":"Given an XML document such as below:\n\n```xml\n<?xml version=\"1.05\" encoding=\"UTF-8\"?>\n \n   \n     \n       Match One \n       \n         \n           1 \n           Team One \n         \n         \n           2 \n           Team Two \n         \n       \n     \n     \n       Match Two \n       \n         \n           2 \n           Team Two \n         \n         \n           3 \n           Team Three \n         \n       \n     \n     \n       Match Three \n       \n         \n           1 \n           Team One \n         \n         \n           3 \n           Team Three \n         \n       \n     \n   \n \n```\nWe can do the following:\n\n```elixir\nimport SweetXml\ndoc = \"...\" # as above\n```\n\nGet the name of the first match:\n\n```elixir\nresult = doc |> xpath(~x\"//matchup/name/text()\") # `sigil_x` for (x)path\nassert result == 'Match One'\n```\n\nGet the XML record of the name of the first match:\n\n```elixir\nresult = doc |> xpath(~x\"//matchup/name\"e) # `e` is the modifier for (e)ntity\nassert result == {:xmlElement, :name, :name, [], {:xmlNamespace, [], []},\n        [matchup: 2, matchups: 2, game: 1], 2, [],\n        [{:xmlText, [name: 2, matchup: 2, matchups: 2, game: 1], 1, [],\n          'Match One', :text}], [],\n        ...}\n```\n\nGet the full list of matchup name:\n\n```elixir\nresult = doc |> xpath(~x\"//matchup/name/text()\"l) # `l` stands for (l)ist\nassert result == ['Match One', 'Match Two', 'Match Three']\n```\n\nGet a list of winner-id by attributes:\n\n```elixir\nresult = doc |> xpath(~x\"//matchup/@winner-id\"l)\nassert result == ['1', '2', '1']\n```\n\nGet a list of matchups with different map structure:\n\n```elixir\nresult = doc |> xpath(\n  ~x\"//matchups/matchup\"l,\n  name: ~x\"./name/text()\",\n  winner: [\n    ~x\".//team/id[.=ancestor::matchup/@winner-id]/..\",\n    name: ~x\"./name/text()\"\n  ]\n)\nassert result == [\n  %{name: 'Match One', winner: %{name: 'Team One'}},\n  %{name: 'Match Two', winner: %{name: 'Team Two'}},\n  %{name: 'Match Three', winner: %{name: 'Team One'}}\n]\n```\n\nOr directly return a mapping of your liking:\n\n```elixir\nresult = doc |> xmap(\n  matchups: [\n    ~x\"//matchups/matchup\"l,\n    name: ~x\"./name/text()\",\n    winner: [\n      ~x\".//team/id[.=ancestor::matchup/@winner-id]/..\",\n      name: ~x\"./name/text()\"\n    ]\n  ],\n  last_matchup: [\n    ~x\"//matchups/matchup[last()]\",\n    name: ~x\"./name/text()\",\n    winner: [\n      ~x\".//team/id[.=ancestor::matchup/@winner-id]/..\",\n      name: ~x\"./name/text()\"\n    ]\n  ]\n)\nassert result == %{\n  matchups: [\n    %{name: 'Match One', winner: %{name: 'Team One'}},\n    %{name: 'Match Two', winner: %{name: 'Team Two'}},\n    %{name: 'Match Three', winner: %{name: 'Team One'}}\n  ],\n  last_matchup: %{name: 'Match Three', winner: %{name: 'Team One'}}\n}\n```","ref":"readme.html#examples"},{"type":"extras","title":"The ~x Sigil - Overview","doc":"Warning ! Because we use `xmerl` internally, only XPath 1.0 paths are handled.\n\nIn the above examples, we used the expression `~x\"//some/path\"` to\ndefine the path. The reason is it allows us to more precisely specify what\nis being returned.\n\n  * `~x\"//some/path\"`\n\n    without any modifiers, `xpath/2` will return the value of the entity if\n    the entity is of type `xmlText`, `xmlAttribute`, `xmlPI`, `xmlComment`\n    as defined in `:xmerl`\n\n  * `~x\"//some/path\"e`\n\n    `e` stands for (e)ntity. This forces `xpath/2` to return the entity with\n    which you can further chain your `xpath/2` call\n\n  * `~x\"//some/path\"l`\n\n    'l' stands for (l)ist. This forces `xpath/2` to return a list. Without\n    `l`, `xpath/2` will only return the first element of the match\n\n  * `~x\"//some/path\"k`\n\n     'k' stands for (k)eyword. This forces `xpath/2` to return a Keyword instead of a Map.\n\n  * `~x\"//some/path\"el` - mix of the above\n\n  * `~x\"//some/path\"s`\n\n    's' stands for (s)tring. This forces `xpath/2` to return the value as\n    string instead of a char list.\n\n  * `~x\"//some/path\"S`\n\n    'S' stands for soft (S)tring. This forces `xpath/2` to return the value as\n    string instead of a char list, but if node content is incompatible with a string,\n    set `\"\"`.\n\n  * `~x\"//some/path\"o`\n\n    'o' stands for (o)ptional. This allows the path to not exist, and will return nil.\n\n  * `~x\"//some/path\"sl` - string list.\n\n  * `~x\"//some/path\"i`\n\n    'i' stands for (i)nteger. This forces `xpath/2` to return the value as\n    integer instead of a char list.\n\n  * `~x//some/path\"I`\n\n    'I' stands for soft (I)nteger. This forces `xpath/2` to return the value as\n    integer instead of a char list, but if node content is incompatible with an integer,\n    set `0`.\n\n  * `~x\"//some/path\"f`\n\n    'f' stands for (f)loat. This forces `xpath/2` to return the value as\n    float instead of a char list.\n\n  * `~x//some/path\"F`\n\n    'F' stands for soft (F)loat. This forces `xpath/2` to return the value as\n    float instead of a char list, but if node content is incompatible with a float,\n    set `0.0`.\n\n  * `~x\"//some/path\"il` - integer list.\n\nIf you use the *optional* modifier `o` together with a *soft* cast modifier\n(uppercase), then the value is set to `nil` when the value is not compatible\nfor instance `~x//some/path/text()\"Fo` return `nil` if the text is not a number.\n\nAlso in the examples section, we always import SweetXml first. This\nmakes `x_sigil` available in the current scope. Without it, instead of using\n`~x`, you can use the `%SweetXpath` struct\n\n```elixir\nassert ~x\"//some/path\"e == %SweetXpath{path: '//some/path', is_value: false, is_list: false, cast_to: false}\n```\n\nNote the use of char_list in the path definition.","ref":"readme.html#the-x-sigil"},{"type":"extras","title":"Namespace support - Overview","doc":"Given a XML document such as below\n\n```xml\n<?xml version=\"1.05\" encoding=\"UTF-8\"?>\n \n   \n     \n       Match One \n       \n         \n           1 \n           Team One \n           5 \n         \n         \n           2 \n           Team Two \n           2 \n         \n       \n     \n   \n \n```\n\nWe can do the following:\n\n```elixir\nimport SweetXml\nxml_str = \"...\" # as above\ndoc = parse(xml_str, namespace_conformant: true)\n```\n\nNote the fact that we explicitly parse the XML with the `namespace_conformant:\ntrue` option. This is needed to allow nodes to be identified in a prefix\nindependent way.\n\nWe can use namespace prefixes of our preference, regardless of what prefix is\nused in the document:\n\n```elixir\nresult = doc\n  |> xpath(~x\"//ff:matchup/ff:name/text()\"\n           |> add_namespace(\"ff\", \"http://example.com/fantasy-league\"))\n\nassert result == 'Match One'\n```\n\nWe can specify multiple namespace prefixes:\n\n```elixir\nresult = doc\n  |> xpath(~x\"//ff:matchup//bb:runs/text()\"\n           |> add_namespace(\"ff\", \"http://example.com/fantasy-league\")\n           |> add_namespace(\"bb\", \"http://example.com/baseball-stats\"))\n\nassert result == '5'\n```","ref":"readme.html#namespace-support"},{"type":"extras","title":"From Chaining to Nesting - Overview","doc":"Here's a brief explanation to how nesting came about.","ref":"readme.html#from-chaining-to-nesting"},{"type":"extras","title":"Chaining - Overview","doc":"Both `xpath` and `xmap` can take an `:xmerl` XML record as the first argument.\nTherefore you can chain calls to these functions like below:\n\n```elixir\ndoc\n|> xpath(~x\"//li\"l)\n|> Enum.map fn (li_node) ->\n  %{\n    name: li_node |> xpath(~x\"./name/text()\"),\n    age: li_node |> xpath(~x\"./age/text()\")\n  }\nend\n```","ref":"readme.html#chaining"},{"type":"extras","title":"Mapping to a structure - Overview","doc":"Since the previous example is such a common use case, SweetXml allows you just\nsimply do the following\n\n```elixir\ndoc\n|> xpath(\n  ~x\"//li\"l,\n  name: ~x\"./name/text()\",\n  age: ~x\"./age/text()\"\n)\n```","ref":"readme.html#mapping-to-a-structure"},{"type":"extras","title":"Nesting - Overview","doc":"But what you want is sometimes more complex than just that, SweetXml thus also\nallows nesting\n\n```elixir\ndoc\n|> xpath(\n  ~x\"//li\"l,\n  name: [\n    ~x\"./name\",\n    first: ~x\"./first/text()\",\n    last: ~x\"./last/text()\"\n  ],\n  age: ~x\"./age/text()\"\n)\n```","ref":"readme.html#nesting"},{"type":"extras","title":"Transform By - Overview","doc":"Sometimes we need to transform the value to what we need, SweetXml supports that\nvia `transform_by/2`\n\n```elixir\ndoc = \"   john  doe   30  \"\n\nresult = doc |> xpath(\n  ~x\"//li\"l,\n  name: [\n    ~x\"./name\",\n    first: ~x\"./first/text()\"s |> transform_by(&String.capitalize/1),\n    last: ~x\"./last/text()\"s |> transform_by(&String.capitalize/1)\n  ],\n  age: ~x\"./age/text()\"i\n)\n\n^result = [%{age: 30, name: %{first: \"John\", last: \"Doe\"}}]\n```\n\nThe same can be used to break parsing code into reusable functions that can be\nused in nesting:\n\n```elixir\ndoc = \"   john  doe   30  \"\n\nparse_name = fn xpath_node ->\n  xpath_node |> xmap(\n    first: ~x\"./first/text()\"s |> transform_by(&String.capitalize/1),\n    last: ~x\"./last/text()\"s |> transform_by(&String.capitalize/1)\n  )\nend\n\nresult = doc |> xpath(\n  ~x\"//li\"l,\n  name: ~x\"./name\" |> transform_by(parse_name),\n  age: ~x\"./age/text()\"i\n)\n\n^result = [%{age: 30, name: %{first: \"John\", last: \"Doe\"}}]\n```\n\nFor more examples, please take a look at the tests and help.","ref":"readme.html#transform-by"},{"type":"extras","title":"Streaming - Overview","doc":"`SweetXml` now also supports streaming in various forms. Here's a sample XML doc.\nNotice the certain lines have XML tags that span multiple lines:\n\n```xml\n<?xml version=\"1.05\" encoding=\"UTF-8\"?>\n \n   \n     XML Parsing \n      Nested Head  \n   \n   \n     Neato €  \n       \n        First  Second\n        Third \n     \n     \n       \n         Forth \n       \n     \n     first star \n   \n \n```","ref":"readme.html#streaming"},{"type":"extras","title":"Working with `File.stream!/1` - Overview","doc":"Working with streams is exactly the same as working with binaries:\n\n```elixir\nFile.stream!(\"file_above.xml\") |> xpath(...)\n```\n\n### `SweetXml` element streaming\n\nOnce you have a file stream, you may not want to work with the entire document to\nsave memory:\n\n```elixir\nfile_stream = File.stream!(\"file_above.xml\")\n\nresult = file_stream\n|> stream_tags([:li, :special_match_key])\n|> Stream.map(fn\n    {_, doc} ->\n      xpath(doc, ~x\"./text()\")\n  end)\n|> Enum.to_list\n\nassert result == ['\\n        First', 'Second\\n      ', 'Third', 'Forth', 'first star']\n```\n\n**Warning:** In case of large document, you may want to use the `discard`\noption to avoid memory leak.\n\n```elixir\nresult = file_stream\n|> stream_tags([:li, :special_match_key], discard: [:li, :special_match_key])\n```","ref":"readme.html#working-with-file-stream-1"},{"type":"extras","title":"Security - Overview","doc":"Whenever you have to deal with some XML that was not generated by your system (untrusted document),\nit is highly recommended that you separate the parsing step from the mapping step, in order to be able\nto prevent some default behavior through options. You can check the doc for `SweetXml.parse/2` for more details.\nThe current recommendations are:\n```\ndoc |> parse(dtd: :none) |> xpath(spec, subspec)\nenum |> stream_tags(tags, dtd: :none)\n```","ref":"readme.html#security"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2014, Frank Liu\n\nSweetXml source code is licensed under the [MIT License](https://github.com/kbrw/sweet_xml/blob/master/LICENSE).\n\n# CONTRIBUTING\n\nHi, and thank you for wanting to contribute.\nPlease refer to the centralized information available at: https://github.com/kbrw#contributing","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}